-------------------------------------------------------------------------------
-- Localized Lua globals.
-------------------------------------------------------------------------------
local _G = getfenv(0)

-- Functions
local pairs = _G.pairs


-- Libraries
local string = _G.string
local table = _G.table


-------------------------------------------------------------------------------
-- AddOn namespace.
-------------------------------------------------------------------------------
local FOLDER_NAME, private = ...

local LibStub = _G.LibStub
local addon = LibStub("AceAddon-3.0"):NewAddon(FOLDER_NAME, "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")

local sorted_pets = {}

local known_pets_by_npc_id = {}
local unknown_pets_by_npc_id = {}

local known_pets_by_name = {}
local unknown_pets_by_name = {}

local pets_by_npc_id = {}

local pet_registry = {}


-------------------------------------------------------------------------------
-- Constants.
-------------------------------------------------------------------------------
local COMPANION_BUTTON_HEIGHT = 46
local MAX_ACTIVE_PETS = 3
local HOOKS_APPLIED = false

local SORT_FIELDS = {
	[_G.LEVEL] = "level",
	[_G.NAME] = "name",
	[_G.TYPE] = "type",
	[_G.PET_BATTLE_STAT_HEALTH] = "max_health",
	[_G.PET_BATTLE_STAT_POWER] = "attack",
	[_G.PET_BATTLE_STAT_QUALITY] = "rarity",
	[_G.PET_BATTLE_STAT_SPEED] = "speed",
}

local SORT_FALLBACKS = {
	level = "name",
	name = "level",
	type = "name",
	rarity = "name",
	max_health = "name",
	attack = "name",
	speed = "name",
}


-------------------------------------------------------------------------------
-- Variables.
-------------------------------------------------------------------------------
local current_sort_field = _G.NAME
local sort_ascending = true

-------------------------------------------------------------------------------
-- Helper functions.
-------------------------------------------------------------------------------
local AcquireTable
local ReleaseTable
do
	local table_cache = {}


	function AcquireTable()
		return table.remove(table_cache) or {}
	end


	function ReleaseTable(tbl)
		if not tbl then
			return
		end
		table.wipe(tbl)
		table.insert(table_cache, tbl)
	end
end -- do block


local ModifyTooltip
do
	local output = {}
	local temp = {}

	function ModifyTooltip(identifier, known_table, unknown_table)
		local known_entries = known_table[identifier]
		local unknown_entries = unknown_table[identifier]
		table.wipe(output)

		if known_entries then
			local shown_header

			for index = 1, #known_entries do
				local pet = known_entries[index]
				table.wipe(temp)

				if not shown_header then
					table.insert(temp, "\n")
					table.insert(temp, pet.name)
					table.insert(temp, _G.HEADER_COLON)

					if #known_entries < MAX_ACTIVE_PETS then
						table.insert(temp, "\n")
						table.insert(temp, "|cff82c5ff")
						table.insert(temp, _G.UNIT_CAPTURABLE)
						table.insert(temp, "|r")
					end
					table.insert(temp, "\n|cffffffff")
					table.insert(temp, _G.BATTLE_PET_CAPTURED)
					table.insert(temp, _G.HEADER_COLON)
					table.insert(temp, "|r\n")
					shown_header = true
				end
				table.insert(temp, _G.ITEM_QUALITY_COLORS[pet.rarity - 1].hex)
				table.insert(temp, "    ")
				table.insert(temp, _G["BATTLE_PET_BREED_QUALITY" .. pet.rarity])
				table.insert(temp, "|r |cffffffff")
				table.insert(temp, _G.PARENS_TEMPLATE:format(_G.TOOLTIP_UNIT_LEVEL:format(pet.level)))
				table.insert(temp, "|r")

				table.insert(output, table.concat(temp, ""))
			end
		elseif unknown_entries then
			local pet = unknown_entries[1]
			table.wipe(temp)

			table.insert(temp, "\n")
			table.insert(temp, pet.name)
			table.insert(temp, _G.HEADER_COLON)
			table.insert(temp, "\n")
			table.insert(temp, "|cff82c5ff")
			table.insert(temp, _G.UNIT_CAPTURABLE)
			table.insert(temp, "|r")

			table.insert(output, table.concat(temp, ""))
		end

		if #output > 0 then
			_G.GameTooltip:AddLine(table.concat(output, "\n"))
		end
	end
end


do
	local previous_tooltip_line
	local entity_registry = {}

	_G.GameTooltip:HookScript("OnUpdate", function(self)
		if not _G.Minimap:IsMouseOver() then
			return
		end
		local tooltip_line = _G["GameTooltipTextLeft1"]:GetText()

		if not tooltip_line or tooltip_line == previous_tooltip_line then
			return
		end
		previous_tooltip_line = tooltip_line
		tooltip_line = tooltip_line:gsub([[|T(.-)|t]], "")

		if tooltip_line:find("\n") then
			local entities = { ("\n"):split(tooltip_line) }
			table.wipe(entity_registry)

			for index = 1, #entities do
				local entity = entities[index]

				if not entity_registry[entity] then
					ModifyTooltip(entity, known_pets_by_name, unknown_pets_by_name)
					entity_registry[entity] = true
				end
			end
		else
			ModifyTooltip(tooltip_line, known_pets_by_name, unknown_pets_by_name)
		end
		_G.GameTooltip:Show()
	end)

	_G.GameTooltip:HookScript("OnHide", function()
		previous_tooltip_line = nil
	end)
end -- do-block


do
	local PET_TEXTURES = {
		"Border",
		"BorderAlive",
	}

	local PET_FONTS = {
		"Level",
		"Name",
	}

	_G.hooksecurefunc("PetBattleUnitFrame_UpdateDisplay", function(frame)
		if not frame.petOwner or not frame.petIndex then
			return
		end
		local r, g, b, hex = _G.GetItemQualityColor(_G.C_PetBattles.GetBreedQuality(frame.petOwner, frame.petIndex) - 1)

		for index = 1, #PET_TEXTURES do
			if frame[PET_TEXTURES[index]] then
				frame[PET_TEXTURES[index]]:SetVertexColor(r, g, b)
			end
		end

		for index = 1, #PET_FONTS do
			local font_string = frame[PET_FONTS[index]]

			if font_string then
				local text = font_string:GetText()

				if text then
					font_string:SetFormattedText("|c%s%s|r", hex, text)
				end
			end
		end
	end)
end -- do-block


local function FilterDropDown_Initialize(self, level)
	local info = _G.UIDropDownMenu_CreateInfo()
	info.keepShownOnClick = true

	if level == 1 then
		info.text = _G.COLLECTED
		info.func = function(_, _, _, value)
			_G.C_PetJournal.SetFlagFilter(_G.LE_PET_JOURNAL_FLAG_COLLECTED, value)

			if (value) then
				_G.UIDropDownMenu_EnableButton(1, 2)
			else
				_G.UIDropDownMenu_DisableButton(1, 2)
			end
		end
		info.checked = not _G.C_PetJournal.IsFlagFiltered(_G.LE_PET_JOURNAL_FLAG_COLLECTED)
		info.isNotRadio = true
		_G.UIDropDownMenu_AddButton(info, level)

		info.text = _G.FAVORITES_FILTER
		info.func = function(_, _, _, value)
			_G.C_PetJournal.SetFlagFilter(_G.LE_PET_JOURNAL_FLAG_FAVORITES, value)
		end
		info.disabled = not info.checked or info.checked ~= true
		info.checked = not _G.C_PetJournal.IsFlagFiltered(_G.LE_PET_JOURNAL_FLAG_FAVORITES)
		info.isNotRadio = true
		info.leftPadding = 16
		_G.UIDropDownMenu_AddButton(info, level)

		info.leftPadding = 0
		info.disabled = nil

		info.text = _G.NOT_COLLECTED
		info.func = function(_, _, _, value)
			_G.C_PetJournal.SetFlagFilter(_G.LE_PET_JOURNAL_FLAG_NOT_COLLECTED, value)
		end
		info.checked = not _G.C_PetJournal.IsFlagFiltered(_G.LE_PET_JOURNAL_FLAG_NOT_COLLECTED)
		info.isNotRadio = true
		_G.UIDropDownMenu_AddButton(info, level)

		info.checked = nil
		info.isNotRadio = nil
		info.func = nil
		info.hasArrow = true
		info.notCheckable = true

		info.text = _G.PET_FAMILIES
		info.value = "FAMILIES"
		_G.UIDropDownMenu_AddButton(info, level)

		info.text = _G.SOURCES
		info.value = "SOURCES"
		_G.UIDropDownMenu_AddButton(info, level)

		info.text = _G.RAID_FRAME_SORT_LABEL
		info.value = "SORTING"
		_G.UIDropDownMenu_AddButton(info, level)
	else --if level == 2 then
		info.hasArrow = false
		info.isNotRadio = true
		info.notCheckable = true

		if _G.UIDROPDOWNMENU_MENU_VALUE == "FAMILIES" then
			info.text = _G.CHECK_ALL
			info.func = function()
				_G.C_PetJournal.AddAllPetTypesFilter()
				_G.UIDropDownMenu_Refresh(_G.PetTheory_FilterDropDown, 1, 2)
			end
			_G.UIDropDownMenu_AddButton(info, level)

			info.text = _G.UNCHECK_ALL
			info.func = function()
				_G.C_PetJournal.ClearAllPetTypesFilter()
				_G.UIDropDownMenu_Refresh(_G.PetTheory_FilterDropDown, 1, 2)
			end
			_G.UIDropDownMenu_AddButton(info, level)

			info.notCheckable = false
			local numTypes = _G.C_PetJournal.GetNumPetTypes()
			for i = 1, numTypes do
				info.text = _G["BATTLE_PET_NAME_" .. i]
				info.func = function(_, _, _, value)
					_G.C_PetJournal.SetPetTypeFilter(i, value)
				end
				info.checked = function() return not _G.C_PetJournal.IsPetTypeFiltered(i) end
				_G.UIDropDownMenu_AddButton(info, level)
			end
		elseif _G.UIDROPDOWNMENU_MENU_VALUE == "SOURCES" then
			info.text = _G.CHECK_ALL
			info.func = function()
				_G.C_PetJournal.AddAllPetSourcesFilter()
				_G.UIDropDownMenu_Refresh(_G.PetTheory_FilterDropDown, 2, 2)
			end
			_G.UIDropDownMenu_AddButton(info, level)

			info.text = _G.UNCHECK_ALL
			info.func = function()
				_G.C_PetJournal.ClearAllPetSourcesFilter()
				_G.UIDropDownMenu_Refresh(_G.PetTheory_FilterDropDown, 2, 2)
			end
			_G.UIDropDownMenu_AddButton(info, level)

			info.notCheckable = false
			local numSources = _G.C_PetJournal.GetNumPetSources()
			for i = 1, numSources do
				info.text = _G["BATTLE_PET_SOURCE_" .. i]
				info.func = function(_, _, _, value)
					_G.C_PetJournal.SetPetSourceFilter(i, value)
				end
				info.checked = function() return not _G.C_PetJournal.IsPetSourceFiltered(i) end
				_G.UIDropDownMenu_AddButton(info, level)
			end
		elseif _G.UIDROPDOWNMENU_MENU_VALUE == "SORTING" then
			info.keepShownOnClick = true
			info.checked = false
			info.isNotRadio = false
			info.notCheckable = false

			for sort_field in pairs(SORT_FIELDS) do
				info.text = sort_field
				info.func = function(_, _, _, value)
					current_sort_field = sort_field
					_G.UIDropDownMenu_Refresh(_G.PetTheory_FilterDropDown, 1, 2)
					private.UpdatePetList()
				end
				info.checked = function()
					return current_sort_field == sort_field
				end
				_G.UIDropDownMenu_AddButton(info, level)
			end
		end
	end
end


local function sort_check_owned(a, b)
	if not a.is_owned and b.is_owned then
		return false
	elseif not b.is_owned and a.is_owned then
		return true
	end
end

local function field_sort(a, b)
	local owned = sort_check_owned(a, b)

	if _G.type(owned) == "boolean" then
		return owned
	end
	local field = SORT_FIELDS[current_sort_field]
	local a_field, b_field = a[field], b[field]
	local ascending = sort_ascending

	if a_field == b_field then
		local fallback = SORT_FALLBACKS[field]
		a_field, b_field = a[fallback], b[fallback]
		ascending = (fallback ~= "name") and ascending or true
	end

	if ascending then
		return a_field and b_field and a_field < b_field
	end
	return a_field and b_field and a_field > b_field
end


local function PopulateSortedList()
	for npc_id, pet_list in pairs(pets_by_npc_id) do
		ReleaseTable(pet_list)
	end
	table.wipe(pets_by_npc_id)
	table.wipe(sorted_pets)

	local num_pets = _G.C_PetJournal.GetNumPets(_G.PetJournal.isWild)

	for pet_index = 1, num_pets do
		local pet_id, _, is_owned, _, level, _, _, name, _, pet_type, creature_id, _, _, _, can_battle = _G.C_PetJournal.GetPetInfoByIndex(pet_index, _G.PetJournal.isWild)
		local _, max_health, attack, speed, rarity = _G.C_PetJournal.GetPetStats(pet_id)

		local identifier = (pet_id ~= 0) and pet_id or name
		local pet = pet_registry[identifier]

		if not pet then
			pet = {
				can_battle = can_battle,
				id = pet_id,
				name = name,
				rarity = rarity,
				type = pet_type,
			}
			pet_registry[identifier] = pet
		end
		pet.attack = attack
		pet.is_owned = is_owned
		pet.journal_index = pet_index
		pet.level = level
		pet.max_health = max_health
		pet.speed = speed

		sorted_pets[#sorted_pets + 1] = pet

		pets_by_npc_id[creature_id] = pets_by_npc_id[creature_id] or AcquireTable()
		pets_by_npc_id[creature_id][#pets_by_npc_id[creature_id] + 1] = pet
	end
	table.wipe(known_pets_by_npc_id)
	table.wipe(unknown_pets_by_npc_id)

	table.wipe(known_pets_by_name)
	table.wipe(unknown_pets_by_name)

	for creature_id, pet_list in pairs(pets_by_npc_id) do
		if pet_list[1].is_owned then
			known_pets_by_npc_id[creature_id] = pet_list
			known_pets_by_name[pet_list[1].name] = pet_list
		else
			unknown_pets_by_npc_id[creature_id] = pet_list
			unknown_pets_by_name[pet_list[1].name] = pet_list
		end
	end
	table.sort(sorted_pets, field_sort)

	if not HOOKS_APPLIED then
		addon:SecureHook("PetJournal_UpdatePetList", private.UpdatePetList)

		addon:SecureHook("PetJournal_UpdatePetCard", function()
			local frame = _G.PetJournalPetCard

			if not frame.petID then
				return
			end
			local health, max_health, power, speed, rarity = _G.C_PetJournal.GetPetStats(_G.PetJournalPetCard.petID)

			if not rarity then
				return
			end
			local color = _G.ITEM_QUALITY_COLORS[rarity - 1]

			-- No idea why Blizzard hides this if not a captured pet or unable to battle, but I'm undoing it.
			frame.QualityFrame.quality:SetText(_G["BATTLE_PET_BREED_QUALITY" .. rarity])
			frame.QualityFrame.quality:SetVertexColor(color.r, color.g, color.b)
			frame.QualityFrame:Show()
		end)

		addon:SecureHook("PetJournal_UpdatePetLoadOut", function()
			for index = 1, MAX_ACTIVE_PETS do
				local plate = _G.PetJournal.Loadout["Pet" .. index]
				local pet_id, _, _, _, locked = _G.C_PetJournal.GetPetLoadOutInfo(index)

				if not locked and pet_id > 0 then
					local hex = _G.ITEM_QUALITY_COLORS[pet_registry[pet_id].rarity - 1].hex

					plate.name:SetFormattedText("%s%s|r", hex, plate.name:GetText())
					plate.level:SetFormattedText("%s%s|r", hex, plate.level:GetText())
				end
			end
		end)

		HOOKS_APPLIED = true
	end
end


local function PetListEntry_OnEnter(self)
	_G.PetJournal_ShowAbilityTooltip(self, self.abilityID, self.speciesID, self.petID)
end


local function PetListEntry_OnLeave(self)
	_G.PetJournal_HideAbilityTooltip(self)
end


function private.UpdatePetList()
	local scroll_frame = _G.PetJournal.listScroll
	local offset = _G.HybridScrollFrame_GetOffset(scroll_frame)
	local pet_buttons = scroll_frame.buttons
	local pet, index

	local is_wild = _G.PetJournal.isWild

	PopulateSortedList()
	scroll_frame:Show()

	local num_pets, num_owned = _G.C_PetJournal.GetNumPets(is_wild)
	_G.PetJournal.PetCount.Count:SetText(num_owned)

	local summoned_pet_id = _G.C_PetJournal.GetSummonedPetID()

	for i = 1, #pet_buttons do
		pet = pet_buttons[i]
		index = offset + i

		pet:SetScript("OnEnter", PetListEntry_OnEnter)
		pet:SetScript("OnLeave", PetListEntry_OnLeave)

		PetListEntry_OnLeave(pet)

		if index <= #sorted_pets then
			local pet_id, species_id, is_owned, custom_name, level, favorite, is_revoked, name, icon, pet_type, creature_id, source_text, description, is_wild_pet, can_battle = _G.C_PetJournal.GetPetInfoByIndex(sorted_pets[index].journal_index, is_wild)

			if custom_name then
				pet.name:SetText(custom_name)
				pet.name:SetHeight(12)
				pet.subName:Show()
				pet.subName:SetText(name .. "Mufasa")
			else
				pet.name:SetText(name)
				pet.name:SetHeight(30)
				pet.subName:Hide()
			end
			pet.icon:SetTexture(icon)
			pet.petTypeIcon:SetTexture(_G.GetPetTypeTexture(pet_type))

			if favorite then
				pet.dragButton.favorite:Show()
			else
				pet.dragButton.favorite:Hide()
			end

			if is_owned then
				local health, maxHealth, attack, speed, rarity = _G.C_PetJournal.GetPetStats(pet_id)
				local colors = _G.ITEM_QUALITY_COLORS[rarity - 1]

				pet.name:SetFormattedText("%s%s|r", colors.hex, pet.name:GetText())
				pet.dragButton.levelBG:SetShown(can_battle)
				pet.dragButton.level:SetShown(can_battle)
				pet.dragButton.level:SetText(level)

				pet.icon:SetDesaturated(0)
				pet.name:SetFontObject("GameFontNormal")
				pet.petTypeIcon:SetShown(can_battle)
				pet.petTypeIcon:SetDesaturated(0)
				pet.dragButton:Enable()
				pet.iconBorder:Show()
				pet.iconBorder:SetVertexColor(colors.r, colors.g, colors.b)

				if health and health <= 0 then
					pet.isDead:Show()
				else
					pet.isDead:Hide()
				end

				if is_revoked then
					pet.dragButton.levelBG:Hide()
					pet.dragButton.level:Hide()
					pet.iconBorder:Hide()
					pet.icon:SetDesaturated(1)
					pet.petTypeIcon:SetDesaturated(1)
					pet.dragButton:Disable()
				end
			else
				pet.dragButton.levelBG:Hide()
				pet.dragButton.level:Hide()
				pet.icon:SetDesaturated(1)
				pet.iconBorder:Hide()
				pet.name:SetFontObject("GameFontDisable")
				pet.petTypeIcon:SetShown(can_battle)
				pet.petTypeIcon:SetDesaturated(1)
				pet.dragButton:Disable()
				pet.isDead:Hide()
			end

			if pet_id and pet_id == summoned_pet_id then
				pet.dragButton.ActiveTexture:Show()
			else
				pet.dragButton.ActiveTexture:Hide()
			end
			pet.petID = pet_id
			pet.speciesID = species_id
			pet.abilityID = _G.PET_BATTLE_PET_TYPE_PASSIVES[pet_type]
			pet.index = sorted_pets[index].journal_index
			pet.owned = is_owned
			pet:Show()

			--Update Petcard Button
			if _G.PetJournalPetCard.petIndex == sorted_pets[index].journal_index then
				pet.selected = true
				pet.selectedTexture:Show()
			else
				pet.selected = false
				pet.selectedTexture:Hide()
			end

			if pet:IsMouseOver() then
				PetListEntry_OnEnter(pet)
			end
		else
			pet:Hide()
		end
	end
	_G.HybridScrollFrame_Update(scroll_frame, num_pets * COMPANION_BUTTON_HEIGHT, scroll_frame:GetHeight())
end


-------------------------------------------------------------------------------
-- Initialization functions.
-------------------------------------------------------------------------------
function addon:OnInitialize()
	self:RegisterEvent("ADDON_LOADED")
end


function addon:OnEnable(event_name)
	-- Trickery to get the the initial pet list.
	self:ScheduleTimer(function()
		local sfx = _G.tonumber(_G.GetCVar("Sound_EnableSFX"))
		_G.SetCVar("Sound_EnableSFX", 0)
		_G.TogglePetJournal(2)
		_G.HideUIPanel(_G.PetJournalParent)
		_G.SetCVar("Sound_EnableSFX", sfx)
		self:ScheduleTimer(PopulateSortedList, 1)
	end, 2)
end


-------------------------------------------------------------------------------
-- Event handlers.
-------------------------------------------------------------------------------
function addon:ADDON_LOADED(event_name, addon_name)
	if addon_name ~= "Blizzard_PetJournal" then
		return
	end
	self:UnregisterEvent("ADDON_LOADED")
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT")

	-------------------------------------------------------------------------------
	-- Replace the PetJournalListScrollFrame with something we can manipulate.
	-------------------------------------------------------------------------------
	local filter_dropdown = _G.CreateFrame("Frame", "PetTheory_FilterDropDown", _G.PetJournal, "UIDropDownMenuTemplate")
	_G.UIDropDownMenu_Initialize(filter_dropdown, FilterDropDown_Initialize, "MENU");

	local filter_button = _G.CreateFrame("Button", "PetTheory_FilterButton", _G.PetJournal, "UIMenuButtonStretchTemplate")
	filter_button:SetSize(70, 22)
	filter_button:SetPoint("TOPRIGHT", _G.PetJournalLeftInset, -28, -9)
	filter_button:SetText(_G.FILTERS)
	filter_button.rightArrow:Show()
	filter_button:SetScript("OnClick", function(self)
		_G.PlaySound("igMainMenuOptionCheckBoxOn")
		_G.ToggleDropDownMenu(1, nil, filter_dropdown, "PetTheory_FilterButton", 74, 15)
	end)
	_G.PetJournalFilterButton:Hide()

	local sort_toggle = _G.CreateFrame("Button", nil, _G.PetJournal)
	sort_toggle:SetSize(24, 24)
	sort_toggle:SetPoint("LEFT", filter_button, "RIGHT", 0, 0)

	sort_toggle:SetScript("OnClick", function(self, button, down)
		sort_ascending = not sort_ascending
		self:SetTexture()
		private.UpdatePetList()
	end)

	function sort_toggle:SetTexture()
		if sort_ascending then
			self:SetNormalTexture([[Interface\CHATFRAME\UI-ChatIcon-ScrollDown-Up]])
			self:SetPushedTexture([[Interface\CHATFRAME\UI-ChatIcon-ScrollDown-Down]])
			self:SetDisabledTexture([[Interface\CHATFRAME\UI-ChatIcon-ScrollDown-Disabled]])
		else
			self:SetNormalTexture([[Interface\CHATFRAME\UI-ChatIcon-ScrollUp-Up]])
			self:SetPushedTexture([[Interface\CHATFRAME\UI-ChatIcon-ScrollUp-Down]])
			self:SetDisabledTexture([[Interface\CHATFRAME\UI-ChatIcon-ScrollUp-Disabled]])
		end
	end

	sort_toggle:SetHighlightTexture([[Interface\CHATFRAME\UI-ChatIcon-BlinkHilight]])
	sort_toggle:SetTexture()

	local scroll_frame = _G.CreateFrame("ScrollFrame", "PetTheory_ScrollFrame", _G.PetJournal, "HybridScrollFrameTemplate")
	scroll_frame:SetPoint("TOPLEFT", _G.PetJournalLeftInset, "TOPLEFT", 3, -36)
	scroll_frame:SetPoint("BOTTOMRIGHT", _G.PetJournalLeftInset, "BOTTOMRIGHT", -2, 5)

	scroll_frame.update = private.UpdatePetList
	_G.PetJournal.listScroll = scroll_frame

	local scroll_bar = _G.CreateFrame("Slider", "PetTheory_ScrollBar", scroll_frame, "HybridScrollBarTrimTemplate")
	scroll_bar:SetPoint("TOPLEFT", scroll_frame, "TOPRIGHT", 4, 20)
	scroll_bar:SetPoint("BOTTOMLEFT", scroll_frame, "BOTTOMRIGHT", 4, 11)

	scroll_bar.trackBG:Show()
	scroll_bar.trackBG:SetVertexColor(0, 0, 0.75)
	scroll_bar.doNotHide = true

	scroll_frame.scrollBar = scroll_bar
	_G.HybridScrollFrame_CreateButtons(scroll_frame, "CompanionListButtonTemplate", 44, 0)
	_G.PetJournalListScrollFrame:Hide()
end


function addon:UPDATE_MOUSEOVER_UNIT()
	if not _G.UnitIsWildBattlePet("mouseover") then
		return
	end
	ModifyTooltip(_G.tonumber(_G.UnitGUID("mouseover"):sub(6, 10), 16), known_pets_by_npc_id, unknown_pets_by_npc_id)
	_G.GameTooltip:Show()
end
